package dev.fitko.fitconnect.integrationtests;

import dev.fitko.fitconnect.integrationtests.condition.EnableIfEnvironmentVariablesAreSet;
import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.api.domain.model.route.Area;
import dev.fitko.fitconnect.api.domain.model.route.Route;
import dev.fitko.fitconnect.client.RoutingClient;
import dev.fitko.fitconnect.client.factory.ClientFactory;
import dev.fitko.fitconnect.client.router.DestinationSearch;
import org.hamcrest.MatcherAssert;
import org.hamcrest.Matchers;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import java.util.List;
import java.util.UUID;

@EnableIfEnvironmentVariablesAreSet
public class RoutingIT extends IntegrationTestBase {

    @Test
    void testFindDestinationsWithRegionalKey() {

        // Given
        final ApplicationConfig config = getConfigWithCredentialsFromEnvironment("TESTING", true);

        final RoutingClient routingClient = ClientFactory.getRoutingClient(config);

        final DestinationSearch search = DestinationSearch.Builder()
                .withLeikaKey("99123456760610")
                .withArs("064350014014")
                .build();

        // When
        final List<Route> routes = routingClient.findDestinations(search);

        // Then
        MatcherAssert.assertThat(routes, Matchers.hasSize(1));
        MatcherAssert.assertThat(routes.get(0).getDestinationId(), Matchers.is(UUID.fromString("d40e7b13-da98-4b09-9e16-bbd61ca81510")));
    }

    @Test
    void testFindDestinationsWithAreaId() {

        // Given
        final ApplicationConfig config = getConfigWithCredentialsFromEnvironment("TESTING", true);

        final RoutingClient routingClient = ClientFactory.getRoutingClient(config);

        final DestinationSearch search = DestinationSearch.Builder()
                .withLeikaKey("99123456760610")
                .withAreaId("931")
                .build();

        // When
        final List<Route> routes = routingClient.findDestinations(search);

        // Then
        MatcherAssert.assertThat(routes, Matchers.hasSize(1));
        MatcherAssert.assertThat(routes.get(0).getDestinationId(), Matchers.is(UUID.fromString("d40e7b13-da98-4b09-9e16-bbd61ca81510")));
    }

    @Test
    void testFindAreaWithMultipleSearchCriteria() {

        // Given
        final ApplicationConfig config = getConfigWithCredentialsFromEnvironment("TESTING", true);

        final RoutingClient routingClient = ClientFactory.getRoutingClient(config);

        // When
        final List<Area> areas = routingClient.findAreas(List.of("Leip*", "04229"), 0, 10);

        // Then
        MatcherAssert.assertThat(areas, Matchers.is(Matchers.not(Matchers.empty())));
        MatcherAssert.assertThat(areas.size(), Matchers.is(Matchers.lessThanOrEqualTo(10)));
        Assertions.assertTrue(areas.stream().anyMatch(area -> area.getName().equals("Leipzig")));
    }

}
